import unittest
from sdif_toolkit.records import FileDescriptionRecord, MeetRecord, SDIFRecord, TeamIDRecord

class TestSDIFRecord(unittest.TestCase):

    def test_init(self):
        line = "A01V3      02Meet Results                  Hy-Tek, Ltd         WMM 7.0Fb Hy-Tek, Ltd         866-456-511103062022                                    MM40    N42"
        self.assertEqual(SDIFRecord(line).code, "A0")

class TestFileDescriptionRecord(unittest.TestCase):

    def test_init(self):
        line = "A01V3      02Meet Results                  Hy-Tek, Ltd         WMM 7.0Fb Hy-Tek, Ltd         866-456-511103062022                                    MM40    N42"
        record = FileDescriptionRecord(line)

        self.assertEqual(SDIFRecord(line).code, "A0")
        self.assertEqual(record.code, "A0")
        self.assertEqual(record.org_code, "1")
        self.assertEqual(record.sdif_version, "V3")
        self.assertEqual(record.file_code, "02")
        self.assertEqual(record.software_name, "Hy-Tek, Ltd")
        self.assertEqual(record.software_version, "WMM 7.0Fb")
        self.assertEqual(record.contact_name, "Hy-Tek, Ltd")
        self.assertEqual(record.contact_phone, "866-456-5111")
        self.assertEqual(record.file_creation_date, "03062022")
        with self.assertRaises(AttributeError):
            record.submitted_by_lsc

class TestMeetRecord(unittest.TestCase):

    def test_init(self):
        line = "B11        2022 SE Indiana Divisional Cha2717 S Morgantown Rd                        Greenwood           IN46143     USA 0304202203062022   0        Y 500   N28"
        record = MeetRecord(line)

        self.assertEqual(SDIFRecord(line).code, "B1")
        self.assertEqual(record.code, "B1")
        self.assertEqual(record.org_code, "1")
        self.assertEqual(record.name, "2022 SE Indiana Divisional Cha")
        self.assertEqual(record.address_1, "2717 S Morgantown Rd")
        self.assertEqual(record.city, "Greenwood")
        self.assertEqual(record.state, "IN")
        self.assertEqual(record.zip, "46143")
        self.assertEqual(record.start_date, "03042022")
        self.assertEqual(record.end_date, "03062022")
        self.assertEqual(record.altitude, "0")
        self.assertEqual(record.course, "Y")
        with self.assertRaises(AttributeError):
            record.address_2

class TestMeetHostRecord(unittest.TestCase):

    def test_init(self):
        raise NotImplementedError

class TestTeamIDRecord(unittest.TestCase):

    def test_init(self):
        line = "C11IN      INFRSTFranklin Regional Swim Team                   654 Walnut St.                              Franklin            IN46131                       N85"
        record = TeamIDRecord(line)

        self.assertEqual(SDIFRecord(line).code, "C1")
        self.assertEqual(record.code, "C1")
        self.assertEqual(record.org_code, "1")
        self.assertEqual(record.team_code, "INFRST")
        self.assertEqual(record.team_name, "Franklin Regional Swim Team")
        self.assertEqual(record.address_1, "654 Walnut St.")
        self.assertEqual(record.city, "Franklin")
        self.assertEqual(record.state, "IN")
        self.assertEqual(record.zip, "46131")
        
        with self.assertRaises(AttributeError):
            record.team_name_abbr
        with self.assertRaises(AttributeError):
            record.address_2
        with self.assertRaises(AttributeError):
            record.region
        with self.assertRaises(AttributeError):
            record.team_code_ext

class TestTeamEntryRecord(unittest.TestCase):

    def test_init(self):
        raise NotImplementedError

class TestIndividualEventRecord(unittest.TestCase):

    def test_init(self):
        raise NotImplementedError

class TestIndividualAdministrativeRecord(unittest.TestCase):

    def test_init(self):
        raise NotImplementedError

class TestIndividualContactRecord(unittest.TestCase):

    def test_init(self):
        raise NotImplementedError

class TestIndividualInformationRecord(unittest.TestCase):

    def test_init(self):
        raise NotImplementedError

class TestRelayEventRecord(unittest.TestCase):

    def test_init(self):
        raise NotImplementedError

class TestRelayNameRecord(unittest.TestCase):

    def test_init(self):
        raise NotImplementedError

class TestSplitsRecord(unittest.TestCase):

    def test_init(self):
        raise NotImplementedError



if __name__ == "__main__":
    unittest.main()


import unittest
from src.sdif_toolkit.time import Time

class TestTimeValidInit(unittest.TestCase):

    def setUp(self) -> None:
        self.expected_minutes = [0, 1, 2, 20, 59]
        self.expected_seconds = [29, 0, 12, 59, 59]
        self.expected_hundredths = [37, 21, 00, 59, 99]
        self.int_input = [2937, 6021, 13200, 125959, 359999]
        self.str_input = ["29.37", "1:00.21", "2:12.00", "20:59.59", "59:59.99"]
        return super().setUp()

    def test_init_int_minutes(self):
        self.assertListEqual([Time(time).minutes for time in self.int_input], self.expected_minutes)

    def test_init_int_seconds(self):
        self.assertListEqual([Time(time).seconds for time in self.int_input], self.expected_seconds)

    def test_init_int_hundredths(self):
        self.assertListEqual([Time(time).hundredths for time in self.int_input], self.expected_hundredths)

    def test_init_str_minutes(self):
        self.assertListEqual([Time(time).minutes for time in self.str_input], self.expected_minutes)

    def test_init_str_seconds(self):
        self.assertListEqual([Time(time).seconds for time in self.str_input], self.expected_seconds)

    def test_init_str_hundredths(self):
        self.assertListEqual([Time(time).hundredths for time in self.str_input], self.expected_hundredths)

class TestTimeSemiValidInit(unittest.TestCase):

    def test_no_semicolon(self):
        time = Time("100.67")
        self.assertEqual(time.minutes, 1)
        self.assertEqual(time.seconds, 0)
        self.assertEqual(time.hundredths, 67)

    def test_no_dot(self):
        time = Time("2:4345")
        self.assertEqual(time.minutes, 2)
        self.assertEqual(time.seconds, 43)
        self.assertEqual(time.hundredths, 45)

    def test_shorthand_hundredths(self):
        time = Time("30.6")
        self.assertEqual(time.minutes, 0)
        self.assertEqual(time.seconds, 30)
        self.assertEqual(time.hundredths, 60)

    def test_no_hundredths(self):
        time = Time("22")
        self.assertEqual(time.minutes, 0)
        self.assertEqual(time.seconds, 22)
        self.assertEqual(time.hundredths, 0)

    def test_semicolon_instead_of_dot(self):
        time = Time("1:03:65")
        self.assertEqual(time.minutes, 1)
        self.assertEqual(time.seconds, 3)
        self.assertEqual(time.hundredths, 65)

class TestTimeInvalidInit(unittest.TestCase):

    def test_not_enough_digits(self):
        with self.assertRaises(ValueError):
            Time("3:3")

    def test_too_many_after_dot(self):
        with self.assertRaises(ValueError):
            Time("1:22.123")

    def test_too_many_between_semicolon_and_dot(self):
        with self.assertRaises(ValueError):
            Time("1:222.00")

    def test_extra_dot(self):
        with self.assertRaises(ValueError):
            Time("1.22.00")
        
class TestTimeStr(unittest.TestCase):

    def setUp(self) -> None:
        self.expected = ["30.60", "1:00.00", "2:15.11"]
        self.int_input = [3060, 6000, 13511]
        self.str_input = ["30.6", "100", "2:15.11"]

    def test_int_to_str(self):
        self.assertListEqual([f"{Time(input)}" for input in self.int_input], self.expected)

    def test_str_to_str(self):
        self.assertListEqual([f"{Time(input)}" for input in self.str_input], self.expected)



import argparse, os, re
from tkinter import filedialog
from typing import Dict, List
from sdif_toolkit.records import RECORD_TYPES, MeetRecord, TeamIDRecord, IndividualEventRecord, SplitsRecord, IndividualInformationRecord, RelayEventRecord
from sdif_toolkit.time import Time

STROKES = {
    "1": "Free",
    "2": "Back",
    "3": "Breast",
    "4": "Fly",
    "5": "IM",
    "6": "Free Relay",
    "7": "Medley Relay"
}

RESULT_TYPES = {
    "p": "Prelims",
    "s": "Swim-Off",
    "f": "Finals"
}

GENDERS_BG = {
    "m": "Boys",
    "f": "Girls"
}

class Result:
    time: Time
    type: str
    splits: List[Time] = []
    split_distance: int
    rank: int
    points: float

    def __repr__(self) -> str:
        return f"{self.time} ({RESULT_TYPES[self.type]})"

class Seed:
    time: Time
    heat: int
    lane: int
    rank: int

    def __repr__(self) -> str:
        return f"{self.time}"

class AgeGroup:
    lower_age: int
    upper_age: int

    def __init__(self, lower_age, upper_age) -> None:
        self.lower_age = lower_age
        self.upper_age = upper_age

    def __repr__(self) -> str:
        if self.lower_age > 0 and self.upper_age < 100:
            return f"{self.lower_age}-{self.upper_age}"
        elif self.lower_age <= 0 and self.upper_age < 100:
            return f"{self.upper_age} & Under"
        elif self.lower_age > 0 and self.upper_age >= 100:
            return f"{self.lower_age} & Over"
        else:
            return "Open"

class Event:
    number: str
    age_group: AgeGroup
    gender: str
    event: str
    seed: Time
    results: Dict[str, Result] = {}

    def __repr__(self) -> str:
        return f"{GENDERS_BG[self.gender]} {self.age_group} {self.event}"

class Swimmer:
    id: str
    first_name: str
    pref_name: str
    middle_name: str
    last_name: str
    birthdate: str
    age: int
    events: List[Event] = []

    def __init__(self, full_name) -> None:
        name = parse_name(full_name)
        self.first_name = name["first_name"]
        if "middle_name" in name.keys():
            self.middle_name = name["middle_name"]
        self.last_name = name["last_name"]
        

    def __repr__(self) -> str:
        return self.full_pref_name

    @property
    def full_name(self):
        if hasattr(self, "middle_name"):
            return f"{self.last_name}, {self.first_name} {self.middle_name}"
        else:
            return f"{self.last_name}, {self.first_name}"

    @property
    def full_pref_name(self):
        first = self.pref_name if hasattr(self, "pref_name") else self.first_name
        return f"{self.last_name}, {first}"

class Team:
    name: str
    code: str
    swimmers: List[Swimmer] = []

    def __repr__(self) -> str:
        return f"{self.code} - {self.name}"

class Meet:
    name: str
    teams: List[Team] = []

    def __repr__(self) -> str:
        return self.name


def parse_name(name: str):
    m = re.match(r"^(?P<last_name>.*), (?P<first_name>.*) (?P<middle_name>[A-Z])$", name)
    if m is not None:
        return m.groupdict()

    m = re.match(r"^(?P<last_name>.*), (?P<first_name>.*)$", name)
    if m is not None:
        return m.groupdict()

    raise ValueError("Name not properly formatted")   
    

def read(input):
    """Read .cl2 file into Meet object"""
    if type(input) is str:
        lines = input.split("\n")
    elif type(input) is list:
        lines = input
    elif type(input) is bytes:
        lines = input.decode("utf-8").split("\n")

    output = []

    for line in lines:
        code = line[0:2]
        
        if code in RECORD_TYPES.keys():
            output.append(RECORD_TYPES[code](line))
    
    output = tuple(output)

    meetRecord = next(record for record in output if type(record) is MeetRecord)

    meet = Meet()
    meet.name = meetRecord.name

    team = None
    swimmers = []
    swimmer = None
    event = None
    splits = None
    prevRecord = None

    for record in output:
        
        if type(record) is TeamIDRecord:
            if team is not None:
                swimmer.events = events
                swimmers.append(swimmer)
                swimmer = None
                team.swimmers = swimmers
                meet.teams.append(team)

            team = Team()
            team.name = record.team_name
            team.code = record.team_code[2:]
            swimmers = []

        elif type(record) is IndividualEventRecord:
            prevRecord = record
            if event is not None and event.event != f"{record.event_distance} {STROKES[record.stroke]}":
                event.results = results
                events.append(event)

            if swimmer is None:
                swimmer = Swimmer(full_name=record.swimmer_name)
                swimmer.age = int(record.swimmer_age)
                events = []
            elif record.swimmer_name != swimmer.full_name:
                swimmer.events = events
                swimmers.append(swimmer)
                swimmer = Swimmer(full_name=record.swimmer_name)
                swimmer.age = int(record.swimmer_age)
                events = []
            
            event = Event()
            event.event = f"{record.event_distance} {STROKES[record.stroke]}"
            event.number = record.event_number
            lower = 0 if record.event_lower_age == "UN" else int(record.event_lower_age)
            upper = 100 if record.event_upper_age == "OV" else int(record.event_upper_age)
            event.age_group = AgeGroup(lower, upper)
            event.gender = record.event_sex.lower()
            results: Dict[str, Result]= {}

            if hasattr(record, "seed_time"):
                time = record.seed_time
                if time != "NS" and time != "DQ":
                    seed = Seed()
                    seed.time = Time(time)
                    event.seed = seed

            if hasattr(record, "prelim_time"):
                time = record.prelim_time
                if time != "NS" and time != "DQ":
                    prelim_result = Result()
                    prelim_result.type = "p"
                    prelim_result.rank = int(record.prelim_rank)
                    prelim_result.time = Time(time)

                    results["p"] = prelim_result
      
            if hasattr(record, "final_time"):
                time = record.final_time
                if time != "NS" and time != "DQ":
                    final_result = Result()
                    final_result.type = "f"
                    final_result.rank = int(record.final_rank)
                    if hasattr(record, "points"):
                        final_result.points = float(record.points)
                    final_result.time = Time(time)

                    results["f"] = final_result

        elif type(record) is IndividualInformationRecord:
            if hasattr(record, "pref_name"):
                swimmer.pref_name = record.pref_name
            swimmer.id = record.swimmer_id

        elif type(record) is RelayEventRecord:
            prevRecord = record
        
        elif type(record) is SplitsRecord and type(prevRecord) is not RelayEventRecord:
            if splits is None:
                splits = []

            for i in range(min(int(record.num_splits) - len(splits), 10)):
                if hasattr(record, f"time_{i + 1}"):
                    time = getattr(record, f"time_{i + 1}")
                    splits.append(Time(time))
            
            if len(splits) == int(record.num_splits):
                if record.swim_code.lower() in results:
                    results[record.swim_code.lower()].splits = splits
                splits = []


    swimmer.events = events
    swimmers.append(swimmer)
    swimmer = None
    team.swimmers = swimmers
    meet.teams.append(team)

    return meet

def is_valid_path(path):
    """Validates path to ensure it is valid in the current file system"""

    if not path:
        raise ValueError("No path given")
    if os.path.isfile(path) or os.path.isdir(path):
        return path
    else:
        raise ValueError(f"Invalid path: {path}")

def parse_args():
    """Get command line arguments"""

    parser = argparse.ArgumentParser(description="Options")
    parser.add_argument('-i', '--input_path', dest='input_path', type=is_valid_path, help="The path of the file or folder to process")

    args = vars(parser.parse_args())

    # Display The Command Line Arguments
    print("## Command Arguments #################################################")
    print("\n".join("{}:{}".format(i, j) for i, j in args.items()))
    print("######################################################################")

    return args

def main():

    args = parse_args()

    input_file = args["input_path"]

    if input_file is None:
        input_file = filedialog.askopenfilename()

    if input_file == '':
        exit()


if __name__ == "__main__":
    main()